import * as React from "react";
import {
  Avatar,
  Button,
  Card,
  Checkbox,
  Grid,
  IconButton,
  List,
  ListItem,
  ListItemAvatar,
  ListItemText,
  Modal,
  TextField,
} from "@mui/material";
import { styled } from "@mui/material/styles";
import SearchIcon from "@mui/icons-material/Search";
import { _products } from "@/shared/_mock/_live";
import Iconify from "@/components/iconify";
import { ILiveProduct } from "@/shared/types/live";

const StyledList = styled(List)({
  overflowY: "auto",
  maxHeight: "calc(100% - 64px)",
  "&::-webkit-scrollbar": {
    width: "10px",
  },
  "&::-webkit-scrollbar-track": {
    boxShadow: "inset 0 0 5px grey",
    borderRadius: "10px",
  },
  "&::-webkit-scrollbar-thumb": {
    background: "darkgrey",
    borderRadius: "10px",
  },
  "&::-webkit-scrollbar-thumb:hover": {
    background: "transparent",
  },
});

type Props = {
  addProduct: (product: ILiveProduct) => void;
  removeProduct: (product: ILiveProduct) => void;
  selectedProducts: ILiveProduct[];
};

const ProductCard = ({ product, isChecked, handleToggle }: any) => (
  <Card variant="outlined" key={product.id} sx={{ marginBottom: 2 }}>
    <ListItem
      secondaryAction={
        <Checkbox
          edge="end"
          checked={isChecked}
          onChange={() => handleToggle(product)}
          inputProps={{
            "aria-labelledby": `checkbox-list-secondary-label-${product.id}`,
          }}
        />
      }
      disablePadding
    >
      <ListItemAvatar>
        <Avatar
          variant="rounded"
          alt={product.title}
          sx={{ height: 100, width: 100 }}
          src={product.image}
        />
      </ListItemAvatar>
      <ListItem
        sx={{
          display: "flex",
          textOverflow: "",
          flexDirection: "column",
          alignItems: "start",
          gap: 1,
        }}
      >
        <ListItemText primary={product.title} />
        <ListItemText secondary={product.reference} />
        <ListItemText primary={`Price: ${product.price}`} />
      </ListItem>
    </ListItem>
  </Card>
);

export default function AddProductPopup({
  addProduct,
  removeProduct,
  selectedProducts,
}: Props) {
  const [open, setOpen] = React.useState(false);
  const [searchTerm, setSearchTerm] = React.useState("");

  const handleOpen = () => setOpen(true);
  const handleClose = () => setOpen(false);

  const handleSearchChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setSearchTerm(event.target.value);
  };

  const handleToggle = (product: ILiveProduct) => {
    const currentIndex = selectedProducts.findIndex(
      (selected) => selected.id === product.id
    );
    if (currentIndex === -1) {
      addProduct(product);
    } else {
      removeProduct(product);
    }
  };

  const filteredProducts = _products.filter(
    (product) =>
      product.title.toLowerCase().includes(searchTerm.toLowerCase()) ||
      product.id.toLowerCase().includes(searchTerm.toLowerCase())
  );

  return (
    <>
      <Button
        onClick={handleOpen}
        variant="contained"
        startIcon={<Iconify icon="mingcute:add-line" />}
      >
        Ajouter produits
      </Button>
      <Modal
        open={open}
        onClose={handleClose}
        aria-labelledby="modal-modal-title"
        aria-describedby="modal-modal-description"
      >
        <Card
          sx={{
            position: "absolute",
            top: "50%",
            left: "50%",
            transform: "translate(-50%, -50%)",
            maxHeight: "80vh",
            maxWidth: "80vw",
            overflow: "hidden",
            display: "flex",
            flexDirection: "column",
            boxShadow: 24,
            p: 2,
          }}
        >
          <TextField
            placeholder="Rechercher un nom de produit ou un SKU"
            InputProps={{
              endAdornment: (
                <IconButton onClick={() => setSearchTerm("")}>
                  <SearchIcon />
                </IconButton>
              ),
            }}
            variant="outlined"
            fullWidth
            value={searchTerm}
            onChange={handleSearchChange}
            sx={{ mb: 2 }}
          />
          <StyledList
            dense
            sx={{
              overflowY: "auto",
              maxHeight: "calc(100% - 64px)",
              p: 2,
            }}
          >
            {filteredProducts.map((product) => {
              const isChecked = selectedProducts.some(
                (selected) => selected.id === product.id
              );
              return (
                <ProductCard
                  key={product.id}
                  product={product}
                  isChecked={isChecked}
                  handleToggle={handleToggle}
                />
              );
            })}
          </StyledList>
        </Card>
      </Modal>
      <StyledList
        dense
        sx={{
          overflowY: "auto",
          maxHeight: "calc(100% - 64px)",
          p: 2,
        }}
      >
        {selectedProducts.map((product) => {
          return (
            <ProductCard
              key={product.id}
              product={product}
              isChecked={true}
              handleToggle={removeProduct}
            />
          );
        })}
      </StyledList>
    </>
  );
}
