import { useRef, useState, useCallback, useEffect } from 'react';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import Collapse from '@mui/material/Collapse';
import Pagination from '@mui/material/Pagination';
import Tooltip from '@mui/material/Tooltip';
import { useBoolean } from '@/hooks/use-boolean';
import Iconify from '@/components/iconify';
import { TableProps } from '@/shared/components/table';
import { IMedia } from '@/shared/types/media';
import MediaManagerPanel from './media-manager-panel';
import MediaManagerActionSelected from './media-manager-action-selected';
import Checkbox from '@mui/material/Checkbox';
import Card from '@mui/material/Card';
import CardMedia from '@mui/material/CardMedia';
import { CardActionArea } from '@mui/material';
import { ConfirmDialog } from '@/shared/components/custom-dialog';
import FileManagerFileDetails from './media-manager-file-details';

// ----------------------------------------------------------------------

type Props = {
  table: TableProps;
  dataFiltered: IMedia[];
  onOpenConfirm: VoidFunction;
  onEditItem: (id: string) => void;
  onDeleteItem: (id: string) => void;
};

export default function MediaManagerGridView({
  table,
  dataFiltered,
  onEditItem,
  onDeleteItem,
  onOpenConfirm,
}: Props) {
  const { selected, onSelectRow: onSelectItem, onSelectAllRows: onSelectAllItems } = table;
  const containerRef = useRef(null);
  const upload = useBoolean();
  const files = useBoolean();
  const confirmDeleteOne = useBoolean();
  const [selectedFileId, setSelectedFileId] = useState<string | null>(null);
  const details = useBoolean();
  const [mediaSelected, setMediaSelected] = useState<IMedia | undefined>();
  const [currentPage, setCurrentPage] = useState<number>(1);
  const itemsPerPage = 20;

  useEffect(() => {
    if (selected.length === 1) {
      setSelectedFileId(selected[0]);
    } else {
      setSelectedFileId(null);
    }
  }, [selected]);

  const handleMediaClick = (media: IMedia) => {
    if (media) {
      setMediaSelected(media);
      details.onTrue();
    }
  };

  // Logique pour obtenir les éléments à afficher sur la page actuelle
  const indexOfLastItem = currentPage * itemsPerPage;
  const indexOfFirstItem = indexOfLastItem - itemsPerPage;
  const currentItems = dataFiltered.slice(indexOfFirstItem, indexOfLastItem);

  const [imageUrl, setImageUrl] = useState('');

  const extractUrl = (url: string) => {
    const match = url.match(/Optional\[(.*)\]/);
    return match ? match[1] : url;
    setImageUrl(extractUrl(url));
  };


  return (
    <>
      <Box ref={containerRef}>
        <MediaManagerPanel
          title="Images"
          subTitle={`images`}
          onOpen={upload.onTrue}
        />

        <Box
          display="grid"
          gridTemplateColumns={{
            xs: 'repeat(1, 1fr)',
            sm: 'repeat(2, 1fr)',
            md: 'repeat(3, 1fr)',
            lg: 'repeat(4, 1fr)',
          }}
          gap={3}
        >
          {currentItems.map((media) => (
            <>
              <Tooltip title={media.name}>
                <Box>
                  <Card sx={{ height:200, boxShadow: selected.includes(media.id) ? '0px 0px 10px 5px #228B22' : '2px solid #ddd'}} onClick={() => handleMediaClick(media)}>
                    <CardActionArea>
                      <CardMedia
                        component="img"
                        image={extractUrl(media.url)}
                        sx={{ height: '100%', objectFit: 'cover', objectPosition: 'center', }}
                      />
                      <Checkbox
                        size="medium"
                        checked={selected.includes(media.id)}
                        onClick={(event) => {
                          event.stopPropagation();
                          onSelectItem(media.id);
                        }}
                        icon={<Iconify icon="eva:radio-button-off-fill" />}
                        checkedIcon={<Iconify icon="eva:checkmark-circle-2-fill" />}
                        sx={{ p: 0.75, position: 'absolute', top: 0, right: 0 }}
                      />
                    </CardActionArea>
                  </Card>
                </Box>
              </Tooltip>
            </>
          ))}
        </Box>

        <Box mt={3} display="flex" justifyContent="center">
          <Pagination
            count={Math.ceil(dataFiltered.length / itemsPerPage)}
            page={currentPage}
            onChange={(event, value) => setCurrentPage(value)}
            variant="outlined"
            shape="rounded"
          />
        </Box>

        {mediaSelected && (
          <FileManagerFileDetails
            item={mediaSelected}
            open={details.value}
            onClose={details.onFalse}
            onEdit={() => onEditItem(mediaSelected.id)}
            onDelete={() => {
              details.onFalse();
              onDeleteItem(mediaSelected.id);
            }}
          />
        )}

        <ConfirmDialog
          open={confirmDeleteOne.value}
          onClose={confirmDeleteOne.onFalse}
          title="Delete"
          content="Are you sure want to delete?"
          action={
            <Button
              variant="contained"
              color="error"
              onClick={() => {
                if (selectedFileId) {
                  onDeleteItem(selectedFileId);
                }
              }}
            >
              Delete
            </Button>
          }
        />

        {/* Actions sélectionnées */}
        {!!selected?.length && (
          <MediaManagerActionSelected
            numSelected={selected.length}
            rowCount={dataFiltered.length}
            selected={selected}
            onSelectAllItems={(checked) =>
              onSelectAllItems(
                checked,
                dataFiltered.map((row) => row.id)
              )
            }
            action={
              <>
                <Button
                  size="small"
                  color="error"
                  variant="contained"
                  startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
                  onClick={onOpenConfirm}
                  sx={{ mr: 1 }}
                >
                  Delete
                </Button>
              </>
            }
          />
        )}
      </Box>
    </>
  );
}
