import { useState, useEffect, useCallback } from 'react';

import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import DialogTitle from '@mui/material/DialogTitle';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import Dialog, { DialogProps } from '@mui/material/Dialog';

import Iconify from '@/components/iconify';
import { Upload } from '@/shared/components/upload';
import { enqueueSnackbar } from 'notistack';
import { useUploadMedias } from '@/shared/api/media';

// ----------------------------------------------------------------------

interface Props extends DialogProps {
  title?: string;
  //
  onCreate?: VoidFunction;
  onUpdate?: VoidFunction;
  //
  mediaName?: string;
  onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
  //
  open: boolean;
  onClose: VoidFunction;
}

export default function NewMediaDialog({
  title = 'Upload Média',
  open,
  onClose,
  //
  onCreate,
  onUpdate,
  //
  mediaName,
  onChangeMediaName,
  ...other
}: Props) {
  const [medias, setMedias] = useState<(File | string)[]>([]);

  useEffect(() => {
    if (!open) {
      setMedias([]);
    }
  }, [open]);

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      const newFiles = acceptedFiles.map((file) =>
        Object.assign(file, {
          preview: URL.createObjectURL(file),
        })
      );

      setMedias([...medias, ...newFiles]);
    },
    [medias]
  );

  const handleUpload = async () => {
    onClose();
    const formData = new FormData();
    medias.forEach(file => formData.append('medias', file));

    try {
      await useUploadMedias(formData);
      if (onCreate) {
        onCreate();
      }
    } catch (error) {
      enqueueSnackbar('Failed to upload this media.', { variant: 'error' });
    }
  };

  const handleRemoveMedia = (inputMedia: File | string) => {
    const filtered = medias.filter((media) => media !== inputMedia);
    setMedias(filtered);
  };

  const handleRemoveAllMedias = () => {
    setMedias([]);
  };

  return (
    <Dialog fullWidth maxWidth="sm" open={open} onClose={onClose} {...other}>
      <DialogTitle sx={{ p: (theme) => theme.spacing(3, 3, 2, 3) }}> {title} </DialogTitle>

      <DialogContent dividers sx={{ pt: 1, pb: 0, border: 'none' }}>
        {(onCreate || onUpdate) && (
          <TextField
            fullWidth
            label="Nom du média"
            value={mediaName}
            onChange={onChangeMediaName}
            sx={{ mb: 3 }}
          />
        )}

        <Upload multiple files={medias} onDrop={handleDrop} onRemove={handleRemoveMedia} />
      </DialogContent>

      <DialogActions>
        <Button
          variant="contained"
          startIcon={<Iconify icon="eva:cloud-upload-fill" />}
          onClick={handleUpload}
        >
          Enregistrer
        </Button>

        {!!medias.length && (
          <Button variant="outlined" color="inherit" onClick={handleRemoveAllMedias}>
            Retirer tout
          </Button>
        )}

        {(onCreate || onUpdate) && (
          <Stack direction="row" justifyContent="flex-end" flexGrow={1}>
            <Button variant="soft" onClick={onCreate || onUpdate}>
              {onUpdate ? 'Save' : 'Create'}
            </Button>
          </Stack>
        )}
      </DialogActions>
    </Dialog>
  );
}
