import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { useMemo, useState, useEffect, useCallback } from "react";
import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Switch from "@mui/material/Switch";
import Divider from "@mui/material/Divider";
import Grid from "@mui/material/Unstable_Grid2";
import CardHeader from "@mui/material/CardHeader";
import Typography from "@mui/material/Typography";
import LoadingButton from "@mui/lab/LoadingButton";
import InputAdornment from "@mui/material/InputAdornment";
import FormControlLabel from "@mui/material/FormControlLabel";
import DeleteIcon from "@mui/icons-material/Delete";
import AddIcon from "@mui/icons-material/Add";
import { Button, IconButton } from "@mui/material";

import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";

import { useResponsive } from "@/hooks/use-responsive";

import { _tags, PRODUCT_CATEGORY_GROUP_OPTIONS } from "@/shared/_mock";

import { useSnackbar } from "@/shared/components/snackbar";

import FormProvider, {
  RHFSelect,
  RHFEditor,
  RHFUpload,
  RHFTextField,
  RHFAutocomplete,
} from "@/shared/components/hook-form";

import { IProductItem } from "@/shared/types/product";
import Variations from "./ProductVariations";

// ----------------------------------------------------------------------

type Props = {
  currentProduct?: IProductItem;
};

export default function ProductNewEditForm({ currentProduct }: Props) {
  const [newAttribute, setNewAttribute] = useState("");
  const [attributeValues, setAttributeValues] = useState("");
  const [selectedAttribute, setSelectedAttribute] = useState("");
  const [parentCategory, setParentCategory] = useState("");
  const [newCategoryName, setNewCategoryName] = useState("");
  const [selectedAttributeValue, setSelectedAttributeValue] = useState("");
  const [siteWebPublication, setSiteWebPublication] = useState(false);
  const [livePublication, setLivePublication] = useState(false);
  const [showVariations, setShowVariations] = useState(true);
  const [attributes, setAttributes] = useState([
    { id: Date.now(), attribute: "", value: "" },
  ]);

  const handleSelectProductType = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    const selectedType = event.target.value as string;
    if (selectedType === "simple") {
      setShowVariations(false);
    } else {
      setShowVariations(true);
    }
  };
  const handleSiteWebPublicationChange = (
    event: React.ChangeEvent<HTMLInputElement>
  ) => {
    setSiteWebPublication(event.target.checked);
  };
  const handleLivePublicationChange = (
    event: React.ChangeEvent<HTMLInputElement>
  ) => {
    setLivePublication(event.target.checked);
  };

  const router = useRouter();

  const mdUp = useResponsive("up", "md");

  const { enqueueSnackbar } = useSnackbar();

  const [includeTaxes, setIncludeTaxes] = useState(false);

  const NewProductSchema = Yup.object().shape({
    name: Yup.string().required("Le nom est requis"),

    images: Yup.array().min(1, "Au moins une image est requise"),

    tags: Yup.array().min(1, "Doit avoir au moins 1 tag"),

    category: Yup.string().required("La catégorie est requise"),

    price: Yup.number().moreThan(0, "Le prix ne doit pas être $0.00"),

    description: Yup.string().required("La description est requise"),

    taxes: Yup.number(),

    newLabel: Yup.object().shape({
      enabled: Yup.boolean(),
      content: Yup.string(),
    }),

    saleLabel: Yup.object().shape({
      enabled: Yup.boolean(),
      content: Yup.string(),
    }),
  });

  const defaultValues = useMemo(
    () => ({
      name: currentProduct?.name || "",
      description: currentProduct?.description || "",
      subDescription: currentProduct?.subDescription || "",
      images: currentProduct?.images || [],
      code: currentProduct?.code || "",
      sku: currentProduct?.sku || "",
      price: currentProduct?.price || 0,
      quantity: currentProduct?.quantity || 0,
      priceSale: currentProduct?.priceSale || 0,
      tags: currentProduct?.tags || [],
      taxes: currentProduct?.taxes || 0,
      gender: currentProduct?.gender || "",
      category: currentProduct?.category || "",
      colors: currentProduct?.colors || [],
      sizes: currentProduct?.sizes || [],
      newLabel: currentProduct?.newLabel || { enabled: false, content: "" },
      saleLabel: currentProduct?.saleLabel || { enabled: false, content: "" },
    }),
    [currentProduct]
  );

  const methods = useForm({
    resolver: yupResolver(NewProductSchema),
    defaultValues,
  });

  const {
    reset,
    watch,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const values = watch();

  useEffect(() => {
    if (currentProduct) {
      reset(defaultValues);
    }
  }, [currentProduct, defaultValues, reset]);

  useEffect(() => {
    if (includeTaxes) {
      setValue("taxes", 0);
    } else {
      setValue("taxes", currentProduct?.taxes || 0);
    }
  }, [currentProduct?.taxes, includeTaxes, setValue]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      enqueueSnackbar(
        currentProduct ? "Mise à jour réussie !" : "Création réussie !"
      );
      router.push(paths.dashboard.product.root);
      console.info("DATA", data);
    } catch (error) {
      console.error(error);
    }
  });

  const handleAttributeChange = (
    event: React.ChangeEvent<{ value: string }>
  ) => {
    const selectedAttribute = event.target.value;
    setSelectedAttribute(selectedAttribute);
    setSelectedAttributeValue("");
  };

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      const files = values.images || [];

      const newFiles = acceptedFiles.map((file) =>
        Object.assign(file, {
          preview: URL.createObjectURL(file),
        })
      );

      setValue("images", [...files, ...newFiles], { shouldValidate: true });
    },
    [setValue, values.images]
  );

  const handleRemoveFile = useCallback(
    (inputFile: File | string) => {
      const filtered =
        values.images && values.images?.filter((file) => file !== inputFile);
      setValue("images", filtered);
    },
    [setValue, values.images]
  );

  const handleRemoveAllFiles = useCallback(() => {
    setValue("images", []);
  }, [setValue]);

  const existingAttributes = ["Taille", "Couleur", "Matériau"];
  type AttributeValuesMap = {
    [key: string]: string[];
  };

  const attributeValuesMap: AttributeValuesMap = {
    Taille: ["Petit", "Moyen", "Grand"],
    Couleur: ["Rouge", "Vert", "Bleu"],
    Matériau: ["Coton", "Polyester", "Laine"],
  };
  const handleAddAttribute = () => {
    if (newAttribute.trim() !== "") {
      console.log("Nouvel attribut ajouté :", newAttribute);
      setNewAttribute("");
    }
  };

  const handleAddNewAttribute = () => {
    setAttributes([
      ...attributes,
      { id: Date.now(), attribute: "", value: "" },
    ]);
  };

  const handleRemoveAttribute = (id: number) => {
    setAttributes(attributes.filter((attr) => attr.id !== id));
  };

  const renderDetails = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Détails
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Titre, brève description, image...
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Détails" />}

          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFTextField name="name" label="Nom du produit" />

            <RHFTextField
              name="subDescription"
              label="Description courte "
              multiline
              rows={4}
            />

            <Stack spacing={1.5}>
              <Typography variant="subtitle2">Description</Typography>
              <RHFEditor
                simple
                onChange={(value) => {
                  console.log(value);
                }}
                name="description"
              />
            </Stack>

            <Stack spacing={1.5}>
              <Typography variant="subtitle2">Images</Typography>
              <RHFUpload
                multiple
                thumbnail
                name="images"
                maxSize={3145728}
                onDrop={handleDrop}
                onRemove={handleRemoveFile}
                onRemoveAll={handleRemoveAllFiles}
                onUpload={() => console.info("ON UPLOAD")}
              />
            </Stack>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderProperties = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Propriétés
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Fonctions et attributs supplémentaires...
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Properties" />}

          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFSelect
              native
              name="productType"
              label="Type de produit"
              InputLabelProps={{ shrink: true }}
              onChange={handleSelectProductType}
            >
              <option value="variable">Produit Variable</option>
              <option value="simple">Produit Simple</option>
            </RHFSelect>
            <Box
              columnGap={2}
              rowGap={3}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                md: "repeat(2, 1fr)",
              }}
            >
              <RHFTextField
                name="price"
                label="Tarif régulier"
                placeholder="0.00"
                type="number"
                InputLabelProps={{ shrink: true }}
                InputProps={{
                  startAdornment: (
                    <InputAdornment position="start">
                      <Box component="span" sx={{ color: "text.disabled" }}>
                        €
                      </Box>
                    </InputAdornment>
                  ),
                }}
              />
              <RHFTextField
                name="priceSale"
                label="Tarif promo"
                placeholder="0.00"
                type="number"
                InputLabelProps={{ shrink: true }}
                InputProps={{
                  startAdornment: (
                    <InputAdornment position="start">
                      <Box component="span" sx={{ color: "text.disabled" }}>
                        €
                      </Box>
                    </InputAdornment>
                  ),
                }}
              />

              <RHFTextField name="UGS" label="UGS produit" />

              <RHFTextField
                name="quantity"
                label="Quantité"
                placeholder="0"
                type="number"
                InputLabelProps={{ shrink: true }}
              />
            </Box>
            <RHFSelect
              native
              name="allowBackorder"
              label="Autoriser les commandes en réapprovisionnement ?"
              InputLabelProps={{ shrink: true }}
            >
              <option value="allow">Autoriser</option>
              <option value="allowWithNotification">
                Autoriser, mais avec notification client
              </option>
              <option value="deny">Ne pas autoriser</option>
            </RHFSelect>
            <RHFTextField
              name="lowStockThreshold"
              label="Seuil de stock faible"
              type="number"
              InputLabelProps={{ shrink: true }}
              placeholder="0"
            />
            <Stack spacing={1}>
              <Stack spacing={3} sx={{ p: 3 }}>
                {attributes.map((attr, index) => (
                  <div
                    key={attr.id}
                    style={{ display: "flex", alignItems: "center" }}
                  >
                    <RHFSelect
                      name={`attributes[${index}].attribute`}
                      label="Attribut"
                      value={attr.attribute}
                      onChange={(e) => {
                        const updatedAttributes = [...attributes];
                        updatedAttributes[index].attribute = e.target.value;
                        setAttributes(updatedAttributes);
                      }}
                      sx={{ flex: 1, marginRight: 1 }}
                    >
                      <option value="" />
                      {existingAttributes.map((attribute) => (
                        <option key={attribute} value={attribute}>
                          {attribute}
                        </option>
                      ))}
                    </RHFSelect>

                    <RHFSelect
                      name={`attributes[${index}].value`}
                      label="Valeur"
                      value={attr.value}
                      onChange={(e) => {
                        const updatedAttributes = [...attributes];
                        updatedAttributes[index].value = e.target.value;
                        setAttributes(updatedAttributes);
                      }}
                      sx={{ flex: 1, marginRight: 1 }}
                      disabled={!selectedAttribute}
                    >
                      <option value="" />
                      {attr.attribute &&
                        attributeValuesMap[attr.attribute]?.map((value) => (
                          <option key={value} value={value}>
                            {value}
                          </option>
                        ))}
                    </RHFSelect>
                    <Button onClick={handleAddNewAttribute} color="primary">
                      <AddIcon fontSize="small" />
                    </Button>
                    {index > 0 && (
                      <IconButton
                        onClick={() => handleRemoveAttribute(attr.id)}
                      >
                        <DeleteIcon fontSize="small" />
                      </IconButton>
                    )}
                  </div>
                ))}
              </Stack>

              <Divider sx={{ borderStyle: "dashed" }} />

              <Typography variant="subtitle2">
                Ajouter un nouvel attribut
              </Typography>

              <Stack direction="row" alignItems="center" spacing={3}>
                <Box
                  columnGap={2}
                  rowGap={3}
                  display="grid"
                  gridTemplateColumns={{
                    xs: "repeat(1, 1fr)",
                    md: "repeat(3, 1fr)",
                  }}
                >
                  <RHFTextField
                    name="newAttribute"
                    label="Nouvel attribut"
                    value={newAttribute}
                    onChange={(e) => setNewAttribute(e.target.value)}
                  />
                  <RHFTextField name="Slug" label="Slug" />
                  <RHFTextField
                    name="attributeValues"
                    label="Valeurs de l'attribut"
                    placeholder="'S', 'M','L','XL'"
                    value={attributeValues}
                    onChange={(e) => setAttributeValues(e.target.value)}
                  />
                </Box>
              </Stack>
              <Button variant="contained" onClick={handleAddAttribute}>
                Ajouter
              </Button>
            </Stack>
            {showVariations && (
              <Variations
                handleDrop={handleDrop}
                handleRemoveFile={handleRemoveFile}
                handleRemoveAllFiles={handleRemoveAllFiles}
                values={values}
              />
            )}
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderCategory = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Catégories et Étiquettes
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Ajouter des catégories et des étiquettes
          </Typography>
        </Grid>
      )}
      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="category" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFSelect
              native
              name="category"
              label="Catégorie"
              value={values.category}
              InputLabelProps={{ shrink: true }}
              onChange={(e) => setValue("category", e.target.value)}
            >
              {PRODUCT_CATEGORY_GROUP_OPTIONS.map((categoryGroup) => (
                <optgroup key={categoryGroup.group} label={categoryGroup.group}>
                  {categoryGroup.classify.map((category) => (
                    <option key={category} value={category}>
                      {category}
                    </option>
                  ))}
                </optgroup>
              ))}

              <option value="select">
                Sélectionner le nom de la catégorie
              </option>
            </RHFSelect>

            <Typography variant="subtitle2">
              Ajouter une nouvelle catégorie
            </Typography>
            <RHFTextField
              name="newCategoryName"
              label="Nom de la nouvelle catégorie"
              value={newCategoryName}
              onChange={(e) => setNewCategoryName(e.target.value)}
            />

            <RHFSelect
              native
              name="parentCategory"
              label="Catégorie parente (facultatif)"
              value={parentCategory}
              InputLabelProps={{ shrink: true }}
              onChange={(e) => setParentCategory(e.target.value)}
            >
              {PRODUCT_CATEGORY_GROUP_OPTIONS.map((categoryGroup) => (
                <optgroup key={categoryGroup.group} label={categoryGroup.group}>
                  {categoryGroup.classify.map((category) => (
                    <option key={category} value={category}>
                      {category}
                    </option>
                  ))}
                </optgroup>
              ))}
            </RHFSelect>
            <Button variant="contained">Ajouter</Button>
          </Stack>
          <Divider sx={{ borderStyle: "dashed" }} />
          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFAutocomplete
              name="tags"
              label="Étiquettes"
              placeholder="+ Étiquettes"
              multiple
              freeSolo
              options={_tags.map((option) => option)}
              getOptionLabel={(option) => option}
              renderOption={(props, option) => (
                <li {...props} key={option}>
                  {option}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option}
                    label={option}
                    size="small"
                    color="info"
                    variant="soft"
                  />
                ))
              }
            />
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderConfiguration = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Configuration Spésifique
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Choisir une Configuration Spésifique
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Expedition" />}

          <Stack direction="row" alignItems="center" spacing={3}>
            <FormControlLabel
              control={
                <Switch
                  checked={siteWebPublication}
                  onChange={handleSiteWebPublicationChange}
                />
              }
              label="Publication sur le site web"
            />
            <FormControlLabel
              control={
                <Switch
                  checked={livePublication}
                  onChange={handleLivePublicationChange}
                />
              }
              label="Publication sur le live"
            />
            {siteWebPublication && (
              <>
                <Stack spacing={3}>
                  <FormControlLabel
                    control={<Switch />}
                    label="Autorisation précommande"
                  />
                  <FormControlLabel
                    control={<Switch />}
                    label="Définir seuil de précommande"
                  />
                </Stack>
              </>
            )}
            {livePublication && (
              <>
                <Stack spacing={3}>
                  <FormControlLabel
                    control={<Switch />}
                    label="Définir seuil de précommande"
                  />
                  <FormControlLabel
                    control={<Switch />}
                    label="Configuration modification"
                  />
                  <FormControlLabel
                    control={<Switch />}
                    label="Alimenter le stock depuis le stock web"
                  />
                  <FormControlLabel
                    control={<Switch />}
                    label="Autorisation précommande"
                  />
                </Stack>
              </>
            )}
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderExpedition = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Expédition
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Inforamtions sur l'expédition
          </Typography>
        </Grid>
      )}
      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Expedition" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFTextField
              name="Poids"
              variant="outlined"
              label="Poids (kg)"
              InputProps={{
                startAdornment: (
                  <InputAdornment position="start">kg</InputAdornment>
                ),
              }}
            />
            <Typography variant="subtitle2"> Dimensions (cm)</Typography>
            <Box
              columnGap={2}
              rowGap={3}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                md: "repeat(3, 1fr)",
              }}
            >
              <RHFTextField name="Longeur" label="Longeur" />
              <RHFTextField name="Largeur" label="Largeur" />
              <RHFTextField name="Hauteur" label="Hauteur" />
            </Box>
            <Typography variant="subtitle2"> Classe d’expédition</Typography>
            <RHFSelect
              native
              name="Classe d’expédition"
              label="Classe d’expédition"
              InputLabelProps={{ shrink: true }}
            >
              <option value="simple">Classe 1 </option>
              <option value="variable">Classe 2</option>
              <option value="variable">Classe 3</option>
            </RHFSelect>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid xs={12} md={8} sx={{ display: "flex", alignItems: "center" }}>
        <FormControlLabel
          control={<Switch defaultChecked />}
          label="Publier"
          sx={{ flexGrow: 1, pl: 3 }}
        />

        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
        >
          {!currentProduct
            ? "Créer le produit"
            : "Enregistrer les modifications"}
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderDetails}
        {renderProperties}
        {renderCategory}

        {renderExpedition}
        {renderConfiguration}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
