import { useState } from 'react';
import { Control, Controller } from 'react-hook-form';
import { Box, Button, Card, CardHeader, Checkbox, Chip, Container, Grid, IconButton, InputAdornment, Modal, Paper, Stack, Table, TableBody, TableCell, TableContainer, TableHead, TableRow, TextField, Tooltip, Typography, Autocomplete, FormLabel, Input } from '@mui/material';
import { Add as AddIcon, Delete as DeleteIcon, Edit as EditIcon, GetApp as GetAppIcon, HelpOutline as HelpOutlineIcon, Publish as PublishIcon } from '@mui/icons-material';
import { useResponsive, useBoolean } from '@/hooks';
import { useSnackbar } from '@/components/snackbar';
import { useGetShippingClasses } from '@/shared/api/shipping';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import { DeliveryCostColissimo, DeliveryReductionColissimo, ShippingClass } from '@/contexts/types/Shipping';

const modalStyle = {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)',
    width: 600,
    bgcolor: 'background.paper',
    boxShadow: 24,
    p: 4,
    borderRadius: 3,
};

const modalContentStyle = {
    display: 'flex',
    flexDirection: 'column',
    gap: 2,
};

interface DetailsColissimoProps {
    control: Control<any>;
    dataListCostDelivery: DeliveryCostColissimo[];
    setDataListCostDelivery: React.Dispatch<React.SetStateAction<DeliveryCostColissimo[]>>;
    dataListReductionDelivery: DeliveryReductionColissimo[];
    setDataListReductionDelivery: React.Dispatch<React.SetStateAction<DeliveryReductionColissimo[]>>;
}

export default function DetailsColissimo({ control, dataListCostDelivery, setDataListCostDelivery, dataListReductionDelivery, setDataListReductionDelivery }: DetailsColissimoProps) {
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    const { shippingClassData } = useGetShippingClasses();
    const [checkedStillFree, setCheckedStillFree] = useState(false);
    const [checkedConditionFree, setCheckedConditionFree] = useState(false);
    const [formDataCost, setFormDataCost] = useState<DeliveryCostColissimo>({
        id: null,
        fromWeight: null,
        toWeight: null,
        fromPrice: null,
        toPrice: null,
        shippingClass: { id: '', name: '', code: '', description: '' } as ShippingClass,
        cost: null,
    });
    const [formDataReduction, setFormDataReduction] = useState<DeliveryReductionColissimo>({
        id: null,
        nbrProducts: null,
        percentageReduction: null,
    });

    const [openAddCostModal, setOpenAddCostModal] = useState(false);
    const [openEditCostModal, setOpenEditCostModal] = useState(false);
    const [openAddReductionModal, setOpenAddReductionModal] = useState(false);
    const [openEditReductionModal, setOpenEditReductionModal] = useState(false);

    const [selectedFileCost, setSelectedFileCost] = useState<File | null>(null);
    const handleFileCostInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        const file = event.target.files?.[0];
        if (file && file.type === 'text/csv') {
            setSelectedFileCost(file);
        } else {
            setSelectedFileCost(null);
        }
    };

    const confirmDeleteCost = useBoolean();
    const confirmDeleteReduction = useBoolean();

    const handleChangeStillFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedStillFree(event.target.checked);
    };

    const handleChangeConditionFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedConditionFree(event.target.checked);
    };

    const handleAddCostClick = () => {
        setFormDataCost({
            id: null,
            fromWeight: null,
            toWeight: null,
            fromPrice: null,
            toPrice: null,
            shippingClass: { id: '', name: '', code: '', description: '' } as ShippingClass,
            cost: null,
        });
        setOpenAddCostModal(true);
    };

    const handleEditCostClick = (index: number) => {
        const row = dataListCostDelivery[index];
        if (row) {
            setFormDataCost({
                id: row.id,
                fromWeight: row.fromWeight,
                toWeight: row.toWeight,
                fromPrice: row.fromPrice,
                toPrice: row.toPrice,
                shippingClass: row.shippingClass,
                cost: row.cost,
            });
        }
        setOpenEditCostModal(true);
    };

    const handleAddReductionClick = () => {
        setFormDataReduction({
            id: null,
            nbrProducts: null,
            percentageReduction: null,
        });
        setOpenAddReductionModal(true);
    };

    const handleEditReductionClick = (index: number) => {
        const row = dataListReductionDelivery[index];
        if (row) {
            setFormDataReduction({
                id: row.id,
                nbrProducts: row.nbrProducts,
                percentageReduction: row.percentageReduction,
            });
        }
        setOpenEditReductionModal(true);
    };

    const handleSaveCost = () => {
        if (formDataCost.id !== null) {
            const updatedList = dataListCostDelivery.map((item) => (item.id === formDataCost.id ? formDataCost : item));
            setDataListCostDelivery(updatedList);
        } else {
            setDataListCostDelivery([...dataListCostDelivery, { ...formDataCost, id: Date.now().toString(), checked: false }]);
        }
        setOpenAddCostModal(false);
        setOpenEditCostModal(false);
    };

    const handleSaveReduction = () => {
        if (formDataReduction.id !== null) {
            const updatedList = dataListReductionDelivery.map((item) => (item.id === formDataReduction.id ? formDataReduction : item));
            setDataListReductionDelivery(updatedList);
        } else {
            setDataListReductionDelivery([...dataListReductionDelivery, { ...formDataReduction, id: Date.now().toString(), checked: false }]);
        }
        setOpenAddReductionModal(false);
        setOpenEditReductionModal(false);
    };

    const renderAddEditCostModal = (open: boolean, handleClose: () => void, title: string, handleSave: () => void) => (
        <Modal open={open} onClose={handleClose}>
            <Box sx={modalStyle}>
                <Typography variant="h6" align="center" sx={{ mt: 2, mb: 3 }} gutterBottom>
                    {title}
                </Typography>
                <Box sx={modalContentStyle}>
                    <TextField
                        label="À partir du poids (g)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCost.fromWeight ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromWeight: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Jusqu’au poids (g)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCost.toWeight ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toWeight: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="À partir du prix du panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.fromPrice ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromPrice: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Jusqu’au prix panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.toPrice ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toPrice: e.target.value ? Number(e.target.value) : null })}
                    />
                    <Autocomplete
                        options={shippingClassData}
                        getOptionLabel={(option) => option.name}
                        value={formDataCost.shippingClass || null}
                        onChange={(_, value) => setFormDataCost({ ...formDataCost, shippingClass: value || { id: '', name: '', code: '', description: '' } })}
                        renderInput={(params) => (
                            <TextField
                                {...params}
                                label="Classe d'expédition"
                                sx={{ m: 1, width: '30ch' }}
                                InputLabelProps={{ shrink: true }}
                            />
                        )}
                        renderTags={(selected, getTagProps) =>
                            selected.map((option, index) => (
                                <Chip
                                    {...getTagProps({ index })}
                                    key={option.id}
                                    label={option.name}
                                    size="small"
                                    color="success"
                                    variant="soft"
                                />
                            ))
                        }
                    />
                    <TextField
                        label="Prix"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '20ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.cost ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, cost: e.target.value ? Number(e.target.value) : null })}
                    />
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={handleClose} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={handleSave} variant="contained" color="inherit">{title.split(' ')[0]}</Button>
                    </Box>
                </Box>
            </Box>
        </Modal>
    );

    const renderAddEditReductionModal = (open: boolean, handleClose: () => void, title: string, handleSave: () => void) => (
        <Modal open={open} onClose={handleClose}>
            <Box sx={modalStyle}>
                <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>{title}</Typography>
                <Box sx={{ width: '80%', ml: 'auto', mr: 'auto' }}>
                    <TextField
                        label="Nombre de produits"
                        type="number"
                        placeholder="0"
                        sx={{ m: 1 }}
                        InputLabelProps={{ shrink: true }}
                        value={formDataReduction.nbrProducts ?? ''}
                        onChange={(e) => setFormDataReduction({ ...formDataReduction, nbrProducts: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Réduction en pourcentage"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1 }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">%</InputAdornment> }}
                        value={formDataReduction.percentageReduction ?? ''}
                        onChange={(e) => setFormDataReduction({ ...formDataReduction, percentageReduction: e.target.value ? Number(e.target.value) : null })}
                    />
                </Box>
                <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                    <Button onClick={handleClose} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                    <Button onClick={handleSave} variant="contained" color="inherit">{title.split(' ')[0]}</Button>
                </Box>
            </Box>
        </Modal>
    );

    const handleRowCheckboxChange = (index: number, checked: boolean, list: any[], setList: React.Dispatch<React.SetStateAction<any[]>>) => {
        const updatedList = [...list];
        updatedList[index].checked = checked;
        setList(updatedList);
    };

    const handleDeleteSelectedRows = (list: any[], setList: React.Dispatch<React.SetStateAction<any[]>>) => {
        const updatedList = list.filter((item) => !item.checked);
        setList(updatedList);
    };

    const handleDeleteCost = (index: number) => {
        handleRowCheckboxChange(index, true, dataListCostDelivery, setDataListCostDelivery);
        confirmDeleteCost.onTrue();
    };

    const handleDeleteReduction = (index: number) => {
        handleRowCheckboxChange(index, true, dataListReductionDelivery, setDataListReductionDelivery);
        confirmDeleteReduction.onTrue();
    };

    return (
        <Container>
            {mdUp && (
                <Grid xs={12} md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Détails Colissimo
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Saisissez les informations du méthode de livraison de type Colissimo
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Détails" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Grid container alignItems="center" spacing={1}>
                            <Controller
                                name="alwaysFree"
                                control={control}
                                defaultValue={checkedStillFree}
                                render={({ field }) => (
                                    <Checkbox
                                        {...field}
                                        checked={field.value}
                                        onChange={(e) => {
                                            setCheckedStillFree(e.target.checked);
                                            field.onChange(e.target.checked);
                                        }}
                                        color="success"
                                        inputProps={{ 'aria-label': 'primary checkbox' }}
                                    />
                                )}
                            />
                            <Grid>
                                <Typography variant="body2">Toujours gratuit ?</Typography>
                            </Grid>
                        </Grid>
                        <Controller
                            name="nameFreeShipping"
                            control={control}
                            defaultValue=""
                            render={({ field }) => <TextField {...field} label="Nom si la livraison est offerte" />}
                        />
                        <Controller
                            name="shippingClassExcludedList"
                            control={control}
                            defaultValue={[]}
                            render={({ field }) => (
                                <Autocomplete
                                    multiple
                                    disableCloseOnSelect
                                    options={shippingClassData}
                                    getOptionLabel={(option) => option.name}
                                    value={field.value}
                                    onChange={(_, value) => {
                                        field.onChange(value);
                                    }}
                                    isOptionEqualToValue={(option, value) => option.id === value.id}
                                    renderInput={(params) => (
                                        <TextField
                                            {...params}
                                            label="Classes d’expédition exclues"
                                            placeholder="+ Classes d’expédition exclues"
                                        />
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option.id}
                                                label={option.name}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            )}
                        />
                        <Controller
                            name="freeShippingClassList"
                            control={control}
                            defaultValue={[]}
                            render={({ field }) => (
                                <Autocomplete
                                    multiple
                                    disableCloseOnSelect
                                    options={shippingClassData}
                                    getOptionLabel={(option) => option.name}
                                    value={field.value}
                                    onChange={(_, value) => {
                                        field.onChange(value);
                                    }}
                                    isOptionEqualToValue={(option, value) => option.id === value.id}
                                    renderInput={(params) => (
                                        <TextField
                                            {...params}
                                            label="Classes d’expédition gratuites"
                                            placeholder="+ Classes d’expédition gratuites"
                                        />
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option.id}
                                                label={option.name}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            )}
                        />
                        <Grid container alignItems="center" spacing={1}>
                            <Grid md={1} xs={2}>
                                <Controller
                                    name="freeIfOneProductHaveFreeShippingClass"
                                    control={control}
                                    defaultValue={checkedConditionFree}
                                    render={({ field }) => (
                                        <Checkbox
                                            {...field}
                                            checked={field.value}
                                            onChange={(e) => {
                                                setCheckedConditionFree(e.target.checked);
                                                field.onChange(e.target.checked);
                                            }}
                                            color="success"
                                            inputProps={{ 'aria-label': 'primary checkbox' }}
                                        />
                                    )}
                                />
                            </Grid>
                            <Grid md={11} xs={10}>
                                <Typography variant="body2">
                                    Gratuit si au moins un des produits du panier a une des classes d’expédition gratuite ci-dessus
                                </Typography>
                            </Grid>
                        </Grid>
                    </Stack>
                </Card>
            </Grid>

            {mdUp && (
                <Grid md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6">
                        Frais de livraison
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Frais de livraison" />}
                    <Container sx={{ mt: 2 }}>
                        <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                            <Button
                                variant="outlined"
                                size="small"
                                sx={{ color: '#808080', width: '15%' }}
                                onClick={handleAddCostClick}
                                startIcon={<AddIcon />}
                            >
                                Ajouter
                            </Button>
                            <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={() => handleDeleteSelectedRows(dataListCostDelivery, setDataListCostDelivery)} startIcon={<DeleteIcon />}>
                                Supprimer
                            </Button>
                            <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<GetAppIcon />}>
                                Exporter
                            </Button>
                            <Input
                                type="file"
                                id="importFileInput"
                                sx={{ display: 'none' }}
                                onChange={handleFileCostInputChange}
                            />
                            <FormLabel htmlFor="importFileInput" sx={{ width: '20%' }}>
                                <Button size="small" sx={{ color: '#808080' }} component="span">
                                    Choisir un fichier
                                </Button>
                            </FormLabel>
                            {selectedFileCost ? (
                                <Typography sx={{ fontSize: '12px', width: '20%' }}>{selectedFileCost.name}</Typography>
                            ) : (
                                <Typography sx={{ fontSize: '12px', width: '20%' }}>Aucun fichier choisi</Typography>
                            )}
                            <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<PublishIcon />}>
                                Importer
                            </Button>
                            <Tooltip
                                title={
                                    <Typography variant="caption">
                                        Le fichier que vous importez doit obligatoirement un CSV avec les 6 colonnes suivants: min_weight, max_weight, min_price,
                                        max_price, shipping_class, price
                                    </Typography>
                                }
                            >
                                <IconButton sx={{ p: 1 }}>
                                    <HelpOutlineIcon sx={{ fontSize: 16 }} />
                                </IconButton>
                            </Tooltip>
                        </Box>
                    </Container>
                    <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '500px' }}>
                        <Table sx={{ minWidth: 650 }} aria-label="simple table">
                            <TableHead>
                                <TableRow>
                                    <TableCell align="center">
                                        <Checkbox
                                            indeterminate={dataListCostDelivery.length > 0 && dataListCostDelivery.some(item => item.checked)}
                                            checked={dataListCostDelivery.length > 0 && dataListCostDelivery.every(item => item.checked)}
                                            onChange={(event) => {
                                                const checked = event.target.checked;
                                                const updatedList = dataListCostDelivery.map(item => ({ ...item, checked }));
                                                setDataListCostDelivery(updatedList);
                                            }}
                                        />
                                    </TableCell>
                                    <TableCell align="center">À partir du poids (g)</TableCell>
                                    <TableCell align="center">Jusqu’au poids (g)</TableCell>
                                    <TableCell align="center">À partir du prix du panier (EUR €)</TableCell>
                                    <TableCell align="center">Jusqu’au prix panier (EUR €)</TableCell>
                                    <TableCell align="center">Classe d’expédition</TableCell>
                                    <TableCell align="center" width="100px">
                                        Prix (EUR €)
                                    </TableCell>
                                    <TableCell align="center" width="115px">
                                        Actions
                                    </TableCell>
                                </TableRow>
                            </TableHead>
                            <TableBody>
                                {dataListCostDelivery.map((row, index) => (
                                    <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                        <TableCell component="th" scope="row" align="center">
                                            <Checkbox
                                                checked={row.checked || false}
                                                onChange={(event) => handleRowCheckboxChange(index, event.target.checked, dataListCostDelivery, setDataListCostDelivery)}
                                            />
                                        </TableCell>
                                        <TableCell align="center">{row.fromWeight} g</TableCell>
                                        <TableCell align="center">{row.toWeight} g</TableCell>
                                        <TableCell align="center">{row.fromPrice} €</TableCell>
                                        <TableCell align="center">{row.toPrice} €</TableCell>
                                        <TableCell align="center">{row.shippingClass?.name}</TableCell>
                                        <TableCell align="center">{row.cost} €</TableCell>
                                        <TableCell align="center">
                                            <Box>
                                                <IconButton onClick={() => handleEditCostClick(index)}>
                                                    <EditIcon sx={{ color: 'green' }} />
                                                </IconButton>
                                                <IconButton onClick={() => handleDeleteCost(index)}>
                                                    <DeleteIcon sx={{ color: 'red' }} />
                                                </IconButton>
                                            </Box>
                                        </TableCell>
                                    </TableRow>
                                ))}
                            </TableBody>
                        </Table>
                    </TableContainer>
                </Card>
            </Grid>

            <ConfirmDialog
                open={confirmDeleteCost.value}
                onClose={confirmDeleteCost.onFalse}
                title="Supprimer frais"
                content="Voulez-vous vraiment supprimer?"
                action={
                    <Button variant="contained" color="error" onClick={() => handleDeleteSelectedRows(dataListCostDelivery, setDataListCostDelivery)}>
                        Supprimer
                    </Button>
                }
            />

            {mdUp && (
                <Grid md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6" >
                        Réduction sur la livraison
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Réduction sur la livraison" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Container sx={{ mt: 2 }}>
                            <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                                <Button
                                    variant="outlined"
                                    size="small"
                                    sx={{ color: '#808080', width: '15%' }}
                                    onClick={handleAddReductionClick}
                                    startIcon={<AddIcon />}
                                >
                                    Ajouter
                                </Button>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={() => handleDeleteSelectedRows(dataListReductionDelivery, setDataListReductionDelivery)} startIcon={<DeleteIcon />}>
                                    Supprimer
                                </Button>
                            </Box>
                        </Container>
                        <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '435px' }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align="center">
                                            <Checkbox
                                                indeterminate={dataListReductionDelivery.length > 0 && dataListReductionDelivery.some(item => item.checked)}
                                                checked={dataListReductionDelivery.length > 0 && dataListReductionDelivery.every(item => item.checked)}
                                                onChange={(event) => {
                                                    const checked = event.target.checked;
                                                    const updatedList = dataListReductionDelivery.map(item => ({ ...item, checked }));
                                                    setDataListReductionDelivery(updatedList);
                                                }}
                                            />
                                        </TableCell>
                                        <TableCell align="center">Nombre de produits</TableCell>
                                        <TableCell align="center">Réduction en pourcentage</TableCell>
                                        <TableCell align="center">Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataListReductionDelivery.map((row, index) => (
                                        <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                            <TableCell component="th" scope="row" align="center">
                                                <Checkbox
                                                    checked={row.checked || false}
                                                    onChange={(event) => handleRowCheckboxChange(index, event.target.checked, dataListReductionDelivery, setDataListReductionDelivery)}
                                                />
                                            </TableCell>
                                            <TableCell component="th" scope="row" align="center">
                                                {row.nbrProducts}
                                            </TableCell>
                                            <TableCell align="center">{row.percentageReduction} %</TableCell>
                                            <TableCell align="center">
                                                <Box>
                                                    <IconButton onClick={() => handleEditReductionClick(index)}>
                                                        <EditIcon sx={{ color: 'green' }} />
                                                    </IconButton>
                                                    <IconButton onClick={() => handleDeleteReduction(index)}>
                                                        <DeleteIcon sx={{ color: 'red' }} />
                                                    </IconButton>
                                                </Box>
                                            </TableCell>
                                        </TableRow>
                                    ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </Stack>
                </Card>
            </Grid>

            <ConfirmDialog
                open={confirmDeleteReduction.value}
                onClose={confirmDeleteReduction.onFalse}
                title="Supprimer réduction"
                content="Voulez-vous vraiment supprimer?"
                action={
                    <Button variant="contained" color="error" onClick={() => handleDeleteSelectedRows(dataListReductionDelivery, setDataListReductionDelivery)}>
                        Supprimer
                    </Button>
                }
            />

            {renderAddEditCostModal(openAddCostModal, () => setOpenAddCostModal(false), "Ajouter frais de livraison", handleSaveCost)}
            {renderAddEditCostModal(openEditCostModal, () => setOpenEditCostModal(false), "Modifier frais de livraison", handleSaveCost)}
            {renderAddEditReductionModal(openAddReductionModal, () => setOpenAddReductionModal(false), "Ajouter réduction sur la livraison", handleSaveReduction)}
            {renderAddEditReductionModal(openEditReductionModal, () => setOpenEditReductionModal(false), "Modifier réduction sur la livraison", handleSaveReduction)}
        </Container>
    );
}
