"use client";

import isEqual from "lodash/isEqual";
import { useState, useEffect, useCallback } from "react";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Grid from "@mui/material/Grid";
import Card from "@mui/material/Card";
import Box from "@mui/material/Box";
import {
  DataGrid,
  GridColDef,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarQuickFilter,
  GridToolbarColumnsButton,
  GridColumnVisibilityModel,
} from "@mui/x-data-grid";
import { paths } from "@/routes/paths";
import { isAfter, isBetween } from '@/utils/format-time';
import EmptyContent from "@/shared/components/empty-content";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import  { SelectChangeEvent } from "@mui/material/Select";
import {
  RenderCellOrderNumber,
  RenderCellStatus,
  RenderCellShippingMode,
  RenderCellDateCreation,
  RenderCellExpedition,
  RenderCellAdress,
  RenderCellExpeditionNumber,
  RenderCellDelivery
} from "./expeditionMondialRelay-table-row";
import { ExpeditionMondialRelayTableFilters, IOrderMondialRelayItem, IOrderMondialRelayStatus, IOrderMondialRelayTableFilters, IOrderMondialRelayTableFilterValue, MondialRelayMode } from "@/contexts/types/Shipping";
import { orderMondialRelayList } from "@/shared/_mock/_shipping";
import ExpeditionTableToolbar from "./expeditionMondialRelay-table-toolbar";
import ExpeditionMondialRelayTableFiltersResult from "./expeditionMondialRelay-table-filters-result";

// ----------------------------------------------------------------------


const ShippingMode_OPTIONS = [
  { value: MondialRelayMode.MED, label: "24R/MED - livraison en Point Relais®" },
  { value: MondialRelayMode.APM, label: "24R/APM - livraison en Lockers" },
  { value: MondialRelayMode.HOM, label: "HOM - livraison domicile HOME" },
  { value: MondialRelayMode.PRL, label: "(obsolète) 24R - livraison en Point Relais® et lockers" },
  { value: MondialRelayMode.HOC, label: "HOC - livraison HOC (indisponible avec l’offre Start)" },
  { value: MondialRelayMode.HOD, label: "HOD - livraison HOD (indisponible avec l’offre Start)" },
  { value: MondialRelayMode.OMR, label: "Livraison hors Mondial Relay - InPost" }
];

const Status_OPTIONS = [
  { value: IOrderMondialRelayStatus.WaitingForPayment, label: "Attente paiement" },
  { value: IOrderMondialRelayStatus.Pending, label: "En attente" },
  { value: IOrderMondialRelayStatus.InProgress, label: "En cours" },
];


const Etiquette_OPTIONS = [
  { value: "Aller", label: "Aller" },
  { value: "Retour", label: "Retour" },
];

const defaultFilters: ExpeditionMondialRelayTableFilters = {
  shippingMode: [],
  status: [],
  startDate: null,
  endDate: null,
};

const HIDE_COLUMNS = {
  category: false,
};

const HIDE_COLUMNS_TOGGLABLE = ["category", "actions"];

// ----------------------------------------------------------------------

export default function ExpeditionMondialRelayListView() {

  const [orders, setOrders] = useState<IOrderMondialRelayItem[]>(orderMondialRelayList);

  const [tableData, setTableData] = useState<IOrderMondialRelayItem[]>(orderMondialRelayList);

  const [filters, setFilters] = useState(defaultFilters);

  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>(
    []
  );

  const [columnVisibilityModel, setColumnVisibilityModel] =
    useState<GridColumnVisibilityModel>(HIDE_COLUMNS);

  useEffect(() => {
    if (orders.length) {
      setTableData(orders);
    }
  }, [orders]);

  const dateError = isAfter(filters.startDate, filters.endDate);

  const dataFiltered = applyFilter({
    inputData: tableData,
    filters,
    dateError,
  });

  const canReset = !isEqual(defaultFilters, filters);

  const handleFilters = useCallback(
    (name: string, value: IOrderMondialRelayTableFilterValue) => {
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    []
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);


  const columns: GridColDef[] = [
    {
      field: "id",
      headerName: "Commande",
      flex: 1,
      width: 100,
      filterable: true,
      hideable: false,
      renderCell: (params) => <RenderCellOrderNumber params={params} />,
    },
    {
      field: "expeditionNumber",
      headerName: "Expédition",
      minWidth: 170,
      renderCell: (params) => <RenderCellExpeditionNumber params={params} />,
    },
    {
      field: "expedition",
      headerName: "Client",
      minWidth: 100,
      renderCell: (params) => <RenderCellExpedition params={params} />,
    },
    {
      field: "address",
      headerName: "Livrée à",
      minWidth: 170,
      renderCell: (params) => <RenderCellAdress params={params} />,
    },
    {
      field: "dateCreation",
      headerName: "Date",
      minWidth: 140,
      renderCell: (params) => <RenderCellDateCreation params={params} />,
    },
    {
      field: "shippingMode",
      headerName: "Mode d'expédition",
      minWidth: 160,
      renderCell: (params) => <RenderCellShippingMode params={params} />,
    },
    {
      field: "status",
      headerName: "Status",
      width: 130,
      hideable: false,
      renderCell: (params) => <RenderCellStatus params={params} />,
    },
    {
      field: "delivery",
      headerName: "Livraison",
      minWidth: 170,
      renderCell: (params) => <RenderCellDelivery params={params} />,
    },
  ];

  const getTogglableColumns = () =>
    columns
      .filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field))
      .map((column) => column.field);

  const [selectedAction, setSelectedAction] = useState("");

  const handleActionChange = (event: SelectChangeEvent<string>) => {
    setSelectedAction(event.target.value);
  };

  const handleButtonClick = () => {
    console.log("Bouton cliqué !");
  };
  const [anchorEl, setAnchorEl] = useState<HTMLElement | null>(null);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <CustomBreadcrumbs
        heading="Expéditions"
        links={[
          { name: "Dashboard", href: paths.dashboard.root },
          { name: 'Livraison', href: paths.dashboard.shipping.general.root, },
          { name: "Mondial Relay - InPost" },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Grid container alignItems="center" >
        <Grid item md={6} sx={{ mb: 1, mr: 1, textAlign: 'right' }}>
          <Button variant="outlined" color="primary" size="small">REGÉNÉRER LES ÉTIQUETTES</Button>
        </Grid>
        <Grid item md={5} sx={{ mb: 1 }}>
          <Button variant="outlined" color="primary" size="small">METTRE À JOUR LES STATUTS</Button>
        </Grid>
      </Grid>

      <Card>
        <Box
          sx={{
            width: "100%",
            flexGrow: { md: 1 },
            display: { md: "flex" },
            flexDirection: { md: "column" },
          }}
        >
          <DataGrid
            checkboxSelection
            disableRowSelectionOnClick
            rows={dataFiltered}
            columns={columns}
            getRowHeight={() => "auto"}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: {
                paginationModel: { pageSize: 10 },
              },
            }}
            onRowSelectionModelChange={(newSelectionModel) => {
              setSelectedRowIds(newSelectionModel);
            }}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) =>
              setColumnVisibilityModel(newModel)
            }
            slots={{
              toolbar: () => (
                <>
                  <GridToolbarContainer>
                    <ExpeditionTableToolbar
                      filters={filters}
                      onFilters={handleFilters}
                      shippingModeOptions={ShippingMode_OPTIONS}
                      statusOptions={Status_OPTIONS}
                      dateError={dateError}
                    />

                    <GridToolbarQuickFilter />

                    <Stack
                      spacing={1}
                      flexGrow={1}
                      direction="row"
                      alignItems="center"
                      justifyContent="flex-end"
                    >

                      <GridToolbarColumnsButton />
                      <GridToolbarExport />
                    </Stack>
                  </GridToolbarContainer>

                  {canReset && (
                    <ExpeditionMondialRelayTableFiltersResult
                      filters={filters}
                      onFilters={handleFilters}
                      onResetFilters={handleResetFilters}
                      results={dataFiltered.length}
                      sx={{ p: 2.5, pt: 0 }}
                    />
                  )}
                </>
              ),
              noRowsOverlay: () => <EmptyContent title="No Data" />,
              noResultsOverlay: () => <EmptyContent title="No results found" />,
            }}
            slotProps={{
              columnsPanel: {
                getTogglableColumns,
              },
            }}
          />
        </Box>
      </Card>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  filters,
  dateError,

}: {
  inputData: IOrderMondialRelayItem[];
  filters: ExpeditionMondialRelayTableFilters;
  dateError: boolean;
}) {
  const { shippingMode, status, startDate, endDate } = filters;

  if (shippingMode.length) {
    inputData = inputData.filter((order) =>
      shippingMode.includes(order.shippingMode)
    );
  }

  if (status.length) {
    inputData = inputData.filter((order) =>
      status.includes(order.status)
    );
  }

  if (!dateError) {
    if (startDate && endDate) {
      inputData = inputData.filter((order) => isBetween(order.dateCreation, startDate, endDate));
    }
  }

  return inputData;
}
