'use client';

import { useState } from 'react';
import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import Chip from '@mui/material/Chip';
import { useTheme } from '@mui/material/styles';
import TableContainer from '@mui/material/TableContainer';
import { paths } from '@/routes/paths';
import { TableHeadCustom } from '@/shared/components/table';
import { useRouter } from '@/hooks';
import { useBoolean } from '@/hooks';
import { methodMondialRelayList } from '@/shared/_mock/_shipping';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TablePaginationCustom, TableEmptyRows, } from '@/shared/components/table';
import { IMethodMondialRelayItem } from '@/contexts/types/Shipping';
import ConfigShippingMethodRow from './ConfigShippingMethodRow';
import { Typography } from '@mui/material';
import { RHFTextField, RHFAutocomplete } from '@/components/hook-form';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useResponsive } from '@/hooks/use-responsive';
import FormProvider from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';

const TABLE_HEAD = [
  { id: "name", label: "Nom", width: 280 },
  { id: "type", label: "Type", with: 80 },
  { id: "shippingArea", label: "Zone de livraison", width: 250 },
  { id: "shippingMode", label: "Mode de livraison", width: 350 },
];

const Etat_OPTIONS = [
  { value: "En attente", label: "En attente" },
  { value: "En cours", label: "En cours" },
  { value: "Terminée", label: "Terminée" },
];

export default function ConfigShippingMethodTable() {

  const theme = useTheme();

  const settings = useSettingsContext();
  const router = useRouter();
  const table = useTable({ defaultOrderBy: 'id' });
  const confirm = useBoolean();
  const [tableData, setTableData] = useState<IMethodMondialRelayItem[]>(methodMondialRelayList);


  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;


  const notFound = (!tableData.length) || !tableData.length;

  const mdUp = useResponsive('up', 'md');
  const { enqueueSnackbar } = useSnackbar();

  const NewDeliveryRulesSchema = Yup.object().shape({
    enableShippingCalculator: Yup.boolean(),
    hideShippingUntilAddress: Yup.boolean(),
    shippingDestination: Yup.string().oneOf(['defaultDeliveryAdress', 'defaultFacturationAdress', 'forceFacturationAdress']).required('Shipping destination is required')
  });



  const methods = useForm({
    resolver: yupResolver(NewDeliveryRulesSchema),
  });


  const {
    handleSubmit,
    formState: { isSubmitting },
  } = methods;



  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      enqueueSnackbar('Success config!', { variant: 'success' });
      console.info('Form Data:', data);
    } catch (error) {
      console.error(error);
      enqueueSnackbar('Failed config.', { variant: 'error' });
    }
  });

  const renderBasic = (
    <>
      <Grid xs={12} md={12}>
        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
          <Grid container alignItems="center" justifyContent="space-between" md={5}>
            <Typography variant="h6" sx={{ mb: 1, mt: 2 }}>
              Méthodes de livraison
            </Typography>
            <Typography variant="body2" sx={{ color: 'text.secondary', mb: 3 }}>
              Configuration des méthodes de livraison du Mondial Relay - InPost.
            </Typography>
          </Grid>

          <Card>
            <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>

              <Scrollbar>
                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                  <TableHeadCustom
                    order={table.order}
                    orderBy={table.orderBy}
                    headLabel={TABLE_HEAD}
                    rowCount={tableData.length}
                    onSort={table.onSort}
                  />
                  <TableBody>
                    {tableData
                      .slice(
                        table.page * table.rowsPerPage,
                        table.page * table.rowsPerPage + table.rowsPerPage
                      )
                      .map((row, index) => (
                        <ConfigShippingMethodRow
                          key={row.id}
                          row={row}
                        />
                      ))}

                    <TableEmptyRows
                      height={denseHeight}
                      emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                    />

                    <TableNoData notFound={notFound} />
                  </TableBody>

                </Table>
              </Scrollbar>
            </TableContainer>

            <TablePaginationCustom
              count={tableData.length}
              page={table.page}
              rowsPerPage={table.rowsPerPage}
              onPageChange={table.onChangePage}
              onRowsPerPageChange={table.onChangeRowsPerPage}
              dense={table.dense}
              onChangeDense={table.onChangeDense}
            />
          </Card>

          <Card sx={{ mt: 2, p: 3 }}>
            <Typography>État des commandes</Typography>
            <Grid container alignItems="center" spacing={1} mt={1}>
              <Grid md={3}>
                <Typography variant="subtitle2">État de la commande à la création</Typography>
              </Grid>
              <Grid md={3}>
                <RHFAutocomplete
                  name="etatOrderAtCreation"
                  label="Etat"
                  placeholder="+ Etat"
                  defaultValue={Etat_OPTIONS[0].label}
                  freeSolo
                  disableCloseOnSelect
                  options={Etat_OPTIONS.map(country => country.label)}
                  getOptionLabel={(option) => option}
                  renderOption={(props, option) => (
                    <li {...props} key={option}>
                      {option}
                    </li>
                  )}
                  renderTags={(selected, getTagProps) =>
                    selected.map((option, index) => (
                      <Chip
                        {...getTagProps({ index })}
                        key={option}
                        label={option}
                        size="small"
                        color="success"
                        variant="soft"
                      />
                    ))
                  }
                />
              </Grid>
            </Grid>
            <Grid container alignItems="center" spacing={1} mt={1}>
              <Grid md={3}>
                <Typography variant="subtitle2">État de la commande à la livraison</Typography>
              </Grid>
              <Grid md={3}>
                <RHFAutocomplete
                  name="etatOrderWhenShipped"
                  label="Etat"
                  placeholder="+ Etat"
                  defaultValue={Etat_OPTIONS[0].label}
                  freeSolo
                  disableCloseOnSelect
                  options={Etat_OPTIONS.map(country => country.label)}
                  getOptionLabel={(option) => option}
                  renderOption={(props, option) => (
                    <li {...props} key={option}>
                      {option}
                    </li>
                  )}
                  renderTags={(selected, getTagProps) =>
                    selected.map((option, index) => (
                      <Chip
                        {...getTagProps({ index })}
                        key={option}
                        label={option}
                        size="small"
                        color="success"
                        variant="soft"
                      />
                    ))
                  }
                />
              </Grid>
            </Grid>
          </Card>
        </Container>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >Enregistrer</LoadingButton>

      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderBasic}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
