"use client";

import { useState, useMemo } from "react";
import {
  Box,
  Grid,
  Card,
  Stack,
  Button,
  Typography,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  IconButton,
  TextField,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableRow,
  TablePagination,
  Avatar,
  ListItemText,
  Chip,
  Container,
} from "@mui/material";
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import {
  useTable,
  TableHeadCustom,
  TableSelectedAction,
  getComparator,
} from "@/shared/components/table";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import LoadingButton from "@mui/lab/LoadingButton";
import { tagsDeviation } from "@/contexts/types/deviation";
import { _products, IProduct } from "@/shared/_mock/_cartLive";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { useSettingsContext } from "@/components/settings";
import EditIcon from "@mui/icons-material/Edit";

type ProductTableFilters = {
  name: string;
  color: string[];
  size: string[];
};

const TABLE_HEAD = [
  { id: "Produit", label: "Produit", width: 250 },
  { id: "id", label: "ID", width: 80 },
  { id: "ugs", label: "UGS", width: 80 },
  { id: "Qte", label: "Quantité", width: 80 },
  { id: "prix", label: "Prix", width: 80 },
  { id: "actions", label: "Actions", width: 80 },
];

type Props = {
  products: IProduct[];
  filters: ProductTableFilters;
  onEditRow: (id: string) => void;
};

const ProductTable = ({ products, filters, onEditRow }: Props) => {
  const [selectedRow, setSelectedRow] = useState<string | null>(null);
  const [page, setPage] = useState(0);
  const [rowsPerPage, setRowsPerPage] = useState(5);

  const table = useTable();

  const handleRowClick = (id: string) => {
    setSelectedRow(id === selectedRow ? null : id);
  };

  const handleChangePage = (event: unknown, newPage: number) => {
    setPage(newPage);
  };

  const handleChangeRowsPerPage = (
    event: React.ChangeEvent<HTMLInputElement>
  ) => {
    setRowsPerPage(parseInt(event.target.value, 10));
    setPage(0);
  };

  const dataFiltered = useMemo(
    () =>
      applyFilter({
        inputData: products,
        comparator: getComparator(table.order, table.orderBy),
        filters,
      }),
    [products, table.order, table.orderBy, filters]
  );

  const dataPaginated = useMemo(() => {
    return dataFiltered.slice(
      page * rowsPerPage,
      page * rowsPerPage + rowsPerPage
    );
  }, [dataFiltered, page, rowsPerPage]);

  return (
    <Box>
      <TableContainer>
        <TableSelectedAction
          dense={table.dense}
          numSelected={table.selected.length}
          rowCount={dataFiltered.length}
          onSelectAllRows={(checked) =>
            table.onSelectAllRows(
              checked,
              dataFiltered.map((row) => row.id)
            )
          }
        />
        <Table size={table.dense ? "small" : "medium"} sx={{ minWidth: 960 }}>
          <TableHeadCustom
            order={table.order}
            orderBy={table.orderBy}
            headLabel={TABLE_HEAD}
            rowCount={dataFiltered.length}
            numSelected={table.selected.length}
            onSort={table.onSort}
          />
          <TableBody>
            {dataPaginated.map((product) => (
              <TableRow
                key={product.id}
                selected={product.id === selectedRow}
                onClick={() => handleRowClick(product.id)}
              >
                <TableCell>
                  <Stack direction="row">
                    <Avatar
                      alt={product.title}
                      src={product.image}
                      variant="rounded"
                      sx={{ width: 48, height: 48, mr: 2 }}
                    />
                    <ListItemText
                      primary={
                        <Box
                          component="div"
                          sx={{ typography: "body2", color: "text.disabled" }}
                        >
                          {product.title}
                        </Box>
                      }
                      sx={{ display: "flex", flexDirection: "column" }}
                    />
                  </Stack>
                </TableCell>
                <TableCell>{product.id}</TableCell>
                <TableCell>{product.UGS}</TableCell>
                <TableCell>{product.quantity}</TableCell>
                <TableCell>{product.price} € </TableCell>
                <TableCell>
                  <IconButton onClick={() => onEditRow(product.id)}>
                    <EditIcon />
                  </IconButton>
                </TableCell>
              </TableRow>
            ))}
          </TableBody>
        </Table>
      </TableContainer>
      <TablePagination
        rowsPerPageOptions={[5, 10, 25]}
        component="div"
        count={dataFiltered.length}
        rowsPerPage={rowsPerPage}
        page={page}
        onPageChange={handleChangePage}
        onRowsPerPageChange={handleChangeRowsPerPage}
      />
    </Box>
  );
};

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: IProduct[];
  comparator: (a: any, b: any) => number;
  filters: ProductTableFilters;
}) {
  const { name, color, size } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (product) =>
        product.title.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (color.length) {
    inputData = inputData.filter((product) =>
      product.colors.some((c) => color.includes(c))
    );
  }

  if (size.length) {
    inputData = inputData.filter((product) =>
      product.sizes.some((s) => size.includes(s))
    );
  }

  return inputData;
}

export default function CreateDeviationView() {
  const TagsDeviation = Object.values(tagsDeviation);
  const settings = useSettingsContext();
  const [openDialog, setOpenDialog] = useState(false);
  const [selectedProduct, setSelectedProduct] = useState<IProduct | null>(null);
  const [selectedDeviationTag, setSelectedDeviationTag] = useState<string>("");

  const handleEditRow = (id: string) => {
    const product = _products.find((product) => product.id === id);
    if (product) {
      setSelectedProduct(product);
      setOpenDialog(true);
    }
  };

  const handleCloseDialog = () => {
    setOpenDialog(false);
    setSelectedProduct(null);
    setSelectedDeviationTag("");
  };

  const handleDeviationTagChange = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    setSelectedDeviationTag(event.target.value as string);
  };

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <CustomBreadcrumbs
        heading="Declaration de l'écart"
        links={[
          { name: "Dashboard", href: paths.dashboard.root },
          {
            name: "Declaration d'écart",
            href: paths.dashboard.supplier.SupplierOrder.deviation,
          },
          { name: "Lister" },
        ]}
        sx={{
          mb: { xs: 3, md: 10 },
        }}
      />
      <Card>
        <ProductTable
          products={_products}
          filters={{ name: "", color: [], size: [] }}
          onEditRow={handleEditRow}
        />

        <Dialog
          open={openDialog}
          onClose={handleCloseDialog}
          maxWidth="sm"
          fullWidth
        >
          <DialogTitle>{"Déclarer l'écart"}</DialogTitle>
          <DialogContent>
            <TextField
              autoFocus
              margin="dense"
              id="description"
              label="Description"
              type="text"
              fullWidth
              variant="outlined"
              multiline
              rows={4}
            />
            <TextField
              select
              margin="dense"
              id="deviation-tag"
              label="Tag de Déviation"
              value={selectedDeviationTag}
              onChange={handleDeviationTagChange}
              fullWidth
              variant="outlined"
              SelectProps={{
                native: true,
              }}
            >
              <option value="" />
              {TagsDeviation.map((option) => (
                <option key={option} value={option}>
                  {option}
                </option>
              ))}
            </TextField>
            {selectedDeviationTag === tagsDeviation.MoreTags && (
              <>
                <TextField
                  margin="dense"
                  id="declare-gap"
                  name="Ecart"
                  label="Déclarer l'écart"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  fullWidth
                />
                <Button
                  variant="contained"
                  color="primary"
                  fullWidth
                  component={RouterLink}
                  href={paths.dashboard.supplier.SupplierOrder.new}
                >
                  Créer une commande
                </Button>
              </>
            )}
            {selectedDeviationTag === tagsDeviation.lessTags && (
              <>
                <Typography margin="normal">
                  Les produits sont plus que les étiquettes
                </Typography>
                <TextField
                  margin="dense"
                  id="less-tags-gap"
                  label="Déclarer l'écart"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  fullWidth
                />
              </>
            )}
            {selectedDeviationTag === tagsDeviation.tagsCorrect && (
              <>
                <Typography margin="normal">
                  Produits et quantité exacts
                </Typography>
                <TextField
                  margin="dense"
                  id="no-deviation"
                  name="deviation"
                  label="l'écart est Null"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  fullWidth
                  disabled
                />
              </>
            )}
          </DialogContent>
          <DialogActions>
            <Button onClick={handleCloseDialog} color="secondary">
              Annuler
            </Button>
            <LoadingButton type="submit" variant="contained" color="primary">
              Enregistrer
            </LoadingButton>
          </DialogActions>
        </Dialog>
      </Card>
    </Container>
  );
}
